""" Testing the game state and logic """

import sidi.game as game
import six


class TestState(object):
    """ Testing the game state and moves """

    def setup(self):
        """ Setup """
        pass

    def teardown(self):
        """ Teardown """
        pass

    def test_create_initial_game(self):
        """ Tests releated to the initial state """
        s = game.State(game.consts.Suits.CACTUS)
        assert isinstance(s, game.State)
        cards = set(list(range(36)))
        play_0_hand = set(s.player_0.hand)
        assert len(play_0_hand) == 6
        play_0_top = set(s.player_0.top)
        assert not play_0_hand.intersection(play_0_top)
        assert len(play_0_top) == 6
        play_0_table = set(s.player_0.table)
        assert not play_0_hand.intersection(play_0_table)
        assert not play_0_top.intersection(play_0_table)
        assert len(play_0_table) == 6
        play_1_hand = set(s.player_1.hand)
        play_1_top = set(s.player_1.top)
        play_1_table = set(s.player_1.table)
        set_ = set()
        set_ |= play_0_hand
        set_ |= play_0_top
        set_ |= play_0_table
        set_ |= play_1_hand
        set_ |= play_1_top
        set_ |= play_1_table
        assert cards == set_

    def test_set_get_state(self):
        """ Testing game state encoding to binary string """
        s1 = game.State(game.consts.Suits.CACTUS)
        bs1 = game.State.to_binary_state(s1)
        s2 = game.State.from_binary_state(bs1)
        bs2 = game.State.to_binary_state(s2)
        if six.PY3:
            assert str(type(bs1)) == "<class 'bytes'>"
        else:
            assert type(bs1) == six.types.StringType
        assert bs1 == bs2


class TestCard(object):
    """ Testing card definition and logic """

    def setup(self):
        """ Setup """
        pass

    def teardown(self):
        """ Teardown """
        pass

    def test_eight(self):
        """ Testing single cards """
        card = game.basic_cards[2]
        assert isinstance(card, game.Card)
        assert card.id_ == 8
        assert card.value == 0
        assert card.power == 2
        assert card.trump_value == 0
        assert card.trump_power == 12
        card = game.basic_cards[3]
        assert isinstance(card, game.Card)
        assert card.id_ == 9
        assert card.value == 0
        assert card.power == 3
        assert card.trump_value == 14
        assert card.trump_power == 17

    def test_values(self):
        """ Testing the sum of all cards in the game """
        trump_sum = 0
        sum_      = 0
        for card in game.basic_cards:
            trump_sum += card.trump_value
            sum_      += card.value
        tot_a = trump_sum + 3 * sum_
        assert tot_a == 152
